from .return_class import AbstractApiClass


class UnifiedConnector(AbstractApiClass):
    """
        Lightweight unified connector filter that skips expensive auth transformations.

        Args:
            client (ApiClient): An authenticated API Client instance
            applicationConnectorId (str): The unique ID for the connection.
            databaseConnectorId (str): The unique ID for the connection.
            service (str): The service this connection connects to
            name (str): A user-friendly name for the service
            createdAt (str): When the API key was created
            status (str): The status of the Application Connector
            auth (dict): Non-secret connection information for this connector
            isUserLevel (bool): Whether this is a user-level connector
    """

    def __init__(self, client, applicationConnectorId=None, databaseConnectorId=None, service=None, name=None, createdAt=None, status=None, auth=None, isUserLevel=None):
        super().__init__(client, None)
        self.application_connector_id = applicationConnectorId
        self.database_connector_id = databaseConnectorId
        self.service = service
        self.name = name
        self.created_at = createdAt
        self.status = status
        self.auth = auth
        self.is_user_level = isUserLevel
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'application_connector_id': repr(self.application_connector_id), f'database_connector_id': repr(self.database_connector_id), f'service': repr(
            self.service), f'name': repr(self.name), f'created_at': repr(self.created_at), f'status': repr(self.status), f'auth': repr(self.auth), f'is_user_level': repr(self.is_user_level)}
        class_name = "UnifiedConnector"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'application_connector_id': self.application_connector_id, 'database_connector_id': self.database_connector_id, 'service': self.service,
                'name': self.name, 'created_at': self.created_at, 'status': self.status, 'auth': self.auth, 'is_user_level': self.is_user_level}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from typing import Dict, List, Union

from . import api_class
from .api_class import AgentInterface, AutonomousTriggerType, TrainingConfig, WorkflowGraph
from .code_source import CodeSource
from .database_connector import DatabaseConnector
from .feature_group import FeatureGroup
from .model_version import ModelVersion
from .refresh_schedule import RefreshSchedule
from .return_class import AbstractApiClass


class Model(AbstractApiClass):
    """
        A model

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): The user-friendly name for the model.
            modelId (str): The unique identifier of the model.
            modelConfigType (str): Name of the TrainingConfig class of the model_config.
            modelPredictionConfig (dict): The prediction config options for the model.
            createdAt (str): Date and time at which the model was created.
            projectId (str): The project this model belongs to.
            trainFunctionName (str): Name of the function found in the source code that will be executed to train the model. It is not executed when this function is run.
            predictFunctionName (str): Name of the function found in the source code that will be executed run predictions through model. It is not executed when this function is run.
            predictManyFunctionName (str): Name of the function found in the source code that will be executed to run batch predictions trhough the model.
            initializeFunctionName (str): Name of the function found in the source code to initialize the trained model before using it to make predictions using the model
            trainingInputTables (list): List of feature groups that are supplied to the train function as parameters. Each of the parameters are materialized Dataframes (same type as the functions return value).
            sourceCode (str): Python code used to make the model.
            cpuSize (str): Cpu size specified for the python model training.
            memory (int): Memory in GB specified for the python model training.
            trainingFeatureGroupIds (list of unique string identifiers): The unique identifiers of the feature groups used as the inputs to train this model on.
            algorithmModelConfigs (list[dict]): List of algorithm specific training configs.
            trainingVectorStoreVersions (list): The vector store version IDs used as inputs during training to create this ModelVersion.
            documentRetrievers (list): List of document retrievers use to create this model.
            documentRetrieverIds (list): List of document retriever IDs used to create this model.
            isPythonModel (bool): If this model is handled as python model
            defaultAlgorithm (str): If set, this algorithm will always be used when deploying the model regardless of the model metrics
            customAlgorithmConfigs (dict): User-defined configs for each of the user-defined custom algorithm
            restrictedAlgorithms (dict): User-selected algorithms to train.
            useGpu (bool): If this model uses gpu.
            notebookId (str): The notebook associated with this model.
            trainingRequired (bool): If training is required to keep the model up-to-date.
            latestModelVersion (ModelVersion): The latest model version.
            refreshSchedules (RefreshSchedule): List of refresh schedules that indicate when the next model version will be trained
            codeSource (CodeSource): If a python model, information on the source code
            databaseConnector (DatabaseConnector): Database connector used by the model.
            dataLlmFeatureGroups (FeatureGroup): List of feature groups used by the model for queries
            modelConfig (TrainingConfig): The training config options used to train this model.
    """

    def __init__(self, client, name=None, modelId=None, modelConfigType=None, modelPredictionConfig=None, createdAt=None, projectId=None, trainFunctionName=None, predictFunctionName=None, predictManyFunctionName=None, initializeFunctionName=None, trainingInputTables=None, sourceCode=None, cpuSize=None, memory=None, trainingFeatureGroupIds=None, algorithmModelConfigs=None, trainingVectorStoreVersions=None, documentRetrievers=None, documentRetrieverIds=None, isPythonModel=None, defaultAlgorithm=None, customAlgorithmConfigs=None, restrictedAlgorithms=None, useGpu=None, notebookId=None, trainingRequired=None, refreshSchedules={}, codeSource={}, databaseConnector={}, dataLlmFeatureGroups={}, latestModelVersion={}, modelConfig={}):
        super().__init__(client, modelId)
        self.name = name
        self.model_id = modelId
        self.model_config_type = modelConfigType
        self.model_prediction_config = modelPredictionConfig
        self.created_at = createdAt
        self.project_id = projectId
        self.train_function_name = trainFunctionName
        self.predict_function_name = predictFunctionName
        self.predict_many_function_name = predictManyFunctionName
        self.initialize_function_name = initializeFunctionName
        self.training_input_tables = trainingInputTables
        self.source_code = sourceCode
        self.cpu_size = cpuSize
        self.memory = memory
        self.training_feature_group_ids = trainingFeatureGroupIds
        self.algorithm_model_configs = algorithmModelConfigs
        self.training_vector_store_versions = trainingVectorStoreVersions
        self.document_retrievers = documentRetrievers
        self.document_retriever_ids = documentRetrieverIds
        self.is_python_model = isPythonModel
        self.default_algorithm = defaultAlgorithm
        self.custom_algorithm_configs = customAlgorithmConfigs
        self.restricted_algorithms = restrictedAlgorithms
        self.use_gpu = useGpu
        self.notebook_id = notebookId
        self.training_required = trainingRequired
        self.refresh_schedules = client._build_class(
            RefreshSchedule, refreshSchedules)
        self.code_source = client._build_class(CodeSource, codeSource)
        self.database_connector = client._build_class(
            DatabaseConnector, databaseConnector)
        self.data_llm_feature_groups = client._build_class(
            FeatureGroup, dataLlmFeatureGroups)
        self.latest_model_version = client._build_class(
            ModelVersion, latestModelVersion)
        self.model_config = client._build_class(getattr(
            api_class, modelConfigType, TrainingConfig) if modelConfigType else TrainingConfig, modelConfig)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'model_id': repr(self.model_id), f'model_config_type': repr(self.model_config_type), f'model_prediction_config': repr(self.model_prediction_config), f'created_at': repr(self.created_at), f'project_id': repr(self.project_id), f'train_function_name': repr(self.train_function_name), f'predict_function_name': repr(self.predict_function_name), f'predict_many_function_name': repr(self.predict_many_function_name), f'initialize_function_name': repr(self.initialize_function_name), f'training_input_tables': repr(self.training_input_tables), f'source_code': repr(self.source_code), f'cpu_size': repr(self.cpu_size), f'memory': repr(self.memory), f'training_feature_group_ids': repr(self.training_feature_group_ids), f'algorithm_model_configs': repr(self.algorithm_model_configs), f'training_vector_store_versions': repr(
            self.training_vector_store_versions), f'document_retrievers': repr(self.document_retrievers), f'document_retriever_ids': repr(self.document_retriever_ids), f'is_python_model': repr(self.is_python_model), f'default_algorithm': repr(self.default_algorithm), f'custom_algorithm_configs': repr(self.custom_algorithm_configs), f'restricted_algorithms': repr(self.restricted_algorithms), f'use_gpu': repr(self.use_gpu), f'notebook_id': repr(self.notebook_id), f'training_required': repr(self.training_required), f'refresh_schedules': repr(self.refresh_schedules), f'code_source': repr(self.code_source), f'database_connector': repr(self.database_connector), f'data_llm_feature_groups': repr(self.data_llm_feature_groups), f'latest_model_version': repr(self.latest_model_version), f'model_config': repr(self.model_config)}
        class_name = "Model"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'model_id': self.model_id, 'model_config_type': self.model_config_type, 'model_prediction_config': self.model_prediction_config, 'created_at': self.created_at, 'project_id': self.project_id, 'train_function_name': self.train_function_name, 'predict_function_name': self.predict_function_name, 'predict_many_function_name': self.predict_many_function_name, 'initialize_function_name': self.initialize_function_name, 'training_input_tables': self.training_input_tables, 'source_code': self.source_code, 'cpu_size': self.cpu_size, 'memory': self.memory, 'training_feature_group_ids': self.training_feature_group_ids, 'algorithm_model_configs': self.algorithm_model_configs, 'training_vector_store_versions': self.training_vector_store_versions, 'document_retrievers': self.document_retrievers,
                'document_retriever_ids': self.document_retriever_ids, 'is_python_model': self.is_python_model, 'default_algorithm': self.default_algorithm, 'custom_algorithm_configs': self.custom_algorithm_configs, 'restricted_algorithms': self.restricted_algorithms, 'use_gpu': self.use_gpu, 'notebook_id': self.notebook_id, 'training_required': self.training_required, 'refresh_schedules': self._get_attribute_as_dict(self.refresh_schedules), 'code_source': self._get_attribute_as_dict(self.code_source), 'database_connector': self._get_attribute_as_dict(self.database_connector), 'data_llm_feature_groups': self._get_attribute_as_dict(self.data_llm_feature_groups), 'latest_model_version': self._get_attribute_as_dict(self.latest_model_version), 'model_config': self._get_attribute_as_dict(self.model_config)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}

    def describe_train_test_data_split_feature_group(self):
        """
        Get the train and test data split for a trained model by its unique identifier. This is only supported for models with custom algorithms.

        Args:
            model_id (str): The unique ID of the model. By default, the latest model version will be returned if no version is specified.

        Returns:
            FeatureGroup: The feature group containing the training data and fold information.
        """
        return self.client.describe_train_test_data_split_feature_group(self.model_id)

    def refresh(self):
        """
        Calls describe and refreshes the current object's fields

        Returns:
            Model: The current object
        """
        self.__dict__.update(self.describe().__dict__)
        return self

    def describe(self):
        """
        Retrieves a full description of the specified model.

        Args:
            model_id (str): Unique string identifier associated with the model.

        Returns:
            Model: Description of the model.
        """
        return self.client.describe_model(self.model_id)

    def rename(self, name: str):
        """
        Renames a model

        Args:
            name (str): The new name to assign to the model.
        """
        return self.client.rename_model(self.model_id, name)

    def update_python(self, function_source_code: str = None, train_function_name: str = None, predict_function_name: str = None, predict_many_function_name: str = None, initialize_function_name: str = None, training_input_tables: list = None, cpu_size: str = None, memory: int = None, package_requirements: list = None, use_gpu: bool = None, is_thread_safe: bool = None, training_config: Union[dict, TrainingConfig] = None):
        """
        Updates an existing Python Model using user-provided Python code. If a list of input feature groups is supplied, they will be provided as arguments to the `train` and `predict` functions with the materialized feature groups for those input feature groups.

        This method expects `functionSourceCode` to be a valid language source file which contains the functions named `trainFunctionName` and `predictFunctionName`. `trainFunctionName` returns the ModelVersion that is the result of training the model using `trainFunctionName`. `predictFunctionName` has no well-defined return type, as it returns the prediction made by the `predictFunctionName`, which can be anything.


        Args:
            function_source_code (str): Contents of a valid Python source code file. The source code should contain the functions named `trainFunctionName` and `predictFunctionName`. A list of allowed import and system libraries for each language is specified in the user functions documentation section.
            train_function_name (str): Name of the function found in the source code that will be executed to train the model. It is not executed when this function is run.
            predict_function_name (str): Name of the function found in the source code that will be executed to run predictions through the model. It is not executed when this function is run.
            predict_many_function_name (str): Name of the function found in the source code that will be executed to run batch predictions through the model. It is not executed when this function is run.
            initialize_function_name (str): Name of the function found in the source code to initialize the trained model before using it to make predictions using the model.
            training_input_tables (list): List of feature groups that are supplied to the `train` function as parameters. Each of the parameters are materialized DataFrames (same type as the functions return value).
            cpu_size (str): Size of the CPU for the model training function.
            memory (int): Memory (in GB) for the model training function.
            package_requirements (list): List of package requirement strings. For example: `['numpy==1.2.3', 'pandas>=1.4.0']`.
            use_gpu (bool): Whether this model needs gpu
            is_thread_safe (bool): Whether this model is thread safe
            training_config (TrainingConfig): The training config used to train this model.

        Returns:
            Model: The updated model.
        """
        return self.client.update_python_model(self.model_id, function_source_code, train_function_name, predict_function_name, predict_many_function_name, initialize_function_name, training_input_tables, cpu_size, memory, package_requirements, use_gpu, is_thread_safe, training_config)

    def set_training_config(self, training_config: Union[dict, TrainingConfig], feature_group_ids: List = None):
        """
        Edits the default model training config

        Args:
            training_config (TrainingConfig): The training config used to train this model.
            feature_group_ids (List): The list of feature groups used as input to the model.

        Returns:
            Model: The model object corresponding to the updated training config.
        """
        return self.client.set_model_training_config(self.model_id, training_config, feature_group_ids)

    def set_prediction_params(self, prediction_config: dict):
        """
        Sets the model prediction config for the model

        Args:
            prediction_config (dict): Prediction configuration for the model.

        Returns:
            Model: Model object after the prediction configuration is applied.
        """
        return self.client.set_model_prediction_params(self.model_id, prediction_config)

    def get_metrics(self, model_version: str = None, return_graphs: bool = False, validation: bool = False):
        """
        Retrieves metrics for all the algorithms trained in this model version.

        If only the model's unique identifier (model_id) is specified, the latest trained version of the model (model_version) is used.


        Args:
            model_version (str): Version of the model.
            return_graphs (bool): If true, will return the information used for the graphs on the model metrics page such as PR Curve per label.
            validation (bool): If true, will return the validation metrics instead of the test metrics.

        Returns:
            ModelMetrics: An object containing the model metrics and explanations for what each metric means.
        """
        return self.client.get_model_metrics(self.model_id, model_version, return_graphs, validation)

    def list_versions(self, limit: int = 100, start_after_version: str = None):
        """
        Retrieves a list of versions for a given model.

        Args:
            limit (int): Maximum length of the list of all dataset versions.
            start_after_version (str): Unique string identifier of the version after which the list starts.

        Returns:
            list[ModelVersion]: An array of model versions.
        """
        return self.client.list_model_versions(self.model_id, limit, start_after_version)

    def retrain(self, deployment_ids: List = None, feature_group_ids: List = None, custom_algorithms: list = None, builtin_algorithms: list = None, custom_algorithm_configs: dict = None, cpu_size: str = None, memory: int = None, training_config: Union[dict, TrainingConfig] = None, algorithm_training_configs: list = None):
        """
        Retrains the specified model, with an option to choose the deployments to which the retraining will be deployed.

        Args:
            deployment_ids (List): List of unique string identifiers of deployments to automatically deploy to.
            feature_group_ids (List): List of feature group IDs provided by the user to train the model on.
            custom_algorithms (list): List of user-defined algorithms to train. If not set, will honor the runs from the last time and applicable new custom algorithms.
            builtin_algorithms (list): List of algorithm names or algorithm IDs of Abacus.AI built-in algorithms to train. If not set, will honor the runs from the last time and applicable new built-in algorithms.
            custom_algorithm_configs (dict): User-defined training configs for each custom algorithm.
            cpu_size (str): Size of the CPU for the user-defined algorithms during training.
            memory (int): Memory (in GB) for the user-defined algorithms during training.
            training_config (TrainingConfig): The training config used to train this model.
            algorithm_training_configs (list): List of algorithm specifc training configs that will be part of the model training AutoML run.

        Returns:
            Model: The model that is being retrained.
        """
        return self.client.retrain_model(self.model_id, deployment_ids, feature_group_ids, custom_algorithms, builtin_algorithms, custom_algorithm_configs, cpu_size, memory, training_config, algorithm_training_configs)

    def delete(self):
        """
        Deletes the specified model and all its versions. Models which are currently used in deployments cannot be deleted.

        Args:
            model_id (str): Unique string identifier of the model to delete.
        """
        return self.client.delete_model(self.model_id)

    def set_default_algorithm(self, algorithm: str = None, data_cluster_type: str = None):
        """
        Sets the model's algorithm to default for all new deployments

        Args:
            algorithm (str): Algorithm to pin in the model.
            data_cluster_type (str): Data cluster type to set the lead model for.
        """
        return self.client.set_default_model_algorithm(self.model_id, algorithm, data_cluster_type)

    def list_artifacts_exports(self, limit: int = 25):
        """
        List all the model artifacts exports.

        Args:
            limit (int): Maximum length of the list of all exports.

        Returns:
            list[ModelArtifactsExport]: List of model artifacts exports.
        """
        return self.client.list_model_artifacts_exports(self.model_id, limit)

    def get_training_types_for_deployment(self, model_version: str = None, algorithm: str = None):
        """
        Returns types of models that can be deployed for a given model instance ID.

        Args:
            model_version (str): The unique ID associated with the model version to deploy.
            algorithm (str): The unique ID associated with the algorithm to deploy.

        Returns:
            ModelTrainingTypeForDeployment: Model training types for deployment.
        """
        return self.client.get_model_training_types_for_deployment(self.model_id, model_version, algorithm)

    def update_agent(self, function_source_code: str = None, agent_function_name: str = None, memory: int = None, package_requirements: list = None, description: str = None, enable_binary_input: bool = None, agent_input_schema: dict = None, agent_output_schema: dict = None, workflow_graph: Union[dict, WorkflowGraph] = None, agent_interface: Union[dict, AgentInterface] = None, included_modules: List = None, org_level_connectors: List = None, user_level_connectors: Dict = None, initialize_function_name: str = None, initialize_function_code: str = None, autonomous_trigger_type: Union[dict, AutonomousTriggerType] = None):
        """
        Updates an existing AI Agent. A new version of the agent will be created and published.

        Args:
            memory (int): Memory (in GB) for the agent.
            package_requirements (list): A list of package requirement strings. For example: ['numpy==1.2.3', 'pandas>=1.4.0'].
            description (str): A description of the agent, including its purpose and instructions.
            workflow_graph (WorkflowGraph): The workflow graph for the agent.
            agent_interface (AgentInterface): The interface that the agent will be deployed with.
            included_modules (List): A list of user created custom modules to include in the agent's environment.
            org_level_connectors (List): A list of org level connector ids to be used by the agent.
            user_level_connectors (Dict): A dictionary mapping ApplicationConnectorType keys to lists of OAuth scopes. Each key represents a specific user level application connector, while the value is a list of scopes that define the permissions granted to the application.
            initialize_function_name (str): The name of the function to be used for initialization.
            initialize_function_code (str): The function code to be used for initialization.
            autonomous_trigger_type (AutonomousTriggerType): The type of trigger for autonomous agents. 'SCHEDULE' for periodic execution, 'WEBHOOK' for event-driven execution. Only applicable when agent_interface is AUTONOMOUS.

        Returns:
            Agent: The updated agent.
        """
        return self.client.update_agent(self.model_id, function_source_code, agent_function_name, memory, package_requirements, description, enable_binary_input, agent_input_schema, agent_output_schema, workflow_graph, agent_interface, included_modules, org_level_connectors, user_level_connectors, initialize_function_name, initialize_function_code, autonomous_trigger_type)

    def wait_for_training(self, timeout=None):
        """
        A waiting call until model is trained.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.
        """
        latest_model_version = self.describe().latest_model_version
        if not latest_model_version:
            from .client import ApiException
            raise ApiException(409, 'This model does not have any versions')
        self.latest_model_version = latest_model_version.wait_for_training(
            timeout=timeout)
        return self

    def wait_for_evaluation(self, timeout=None):
        """
        A waiting call until model is evaluated completely.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.
        """
        return self.wait_for_training()

    def wait_for_publish(self, timeout=None):
        """
        A waiting call until agent is published.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.
        """
        return self.wait_for_training()

    def wait_for_full_automl(self, timeout=None):
        """
        A waiting call until full AutoML cycle is completed.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.
        """
        latest_model_version = self.describe().latest_model_version
        if not latest_model_version:
            from .client import ApiException
            raise ApiException(409, 'This model does not have any versions')
        self.latest_model_version = latest_model_version.wait_for_full_automl(
            timeout=timeout)
        return self

    def get_status(self, get_automl_status: bool = False):
        """
        Gets the status of the model training.

        Returns:
            str: A string describing the status of a model training (pending, complete, etc.).
        """
        if get_automl_status:
            return self.client._call_api('describeModel', 'GET', query_params={'modelId': self.model_id, 'waitForFullAutoml': True}, parse_type=Model).latest_model_version.status
        return self.describe().latest_model_version.status

    def create_refresh_policy(self, cron: str):
        """
        To create a refresh policy for a model.

        Args:
            cron (str): A cron style string to set the refresh time.

        Returns:
            RefreshPolicy: The refresh policy object.
        """
        return self.client.create_refresh_policy(self.name, cron, 'MODEL', model_ids=[self.id])

    def list_refresh_policies(self):
        """
        Gets the refresh policies in a list.

        Returns:
            List[RefreshPolicy]: A list of refresh policy objects.
        """
        return self.client.list_refresh_policies(model_ids=[self.id])

    def get_train_test_feature_group_as_pandas(self):
        """
        Get the model train test data split feature group as pandas.

        Returns:
            pandas.Dataframe: A pandas dataframe for the training data with fold column.
        """
        return self.client.describe_train_test_data_split_feature_group(self.model_id).load_as_pandas()



from .return_class import AbstractApiClass


class ComputePointInfo(AbstractApiClass):
    """
        The compute point info of the organization

        Args:
            client (ApiClient): An authenticated API Client instance
            updatedAt (str): The last time the compute point info was updated
            last24HoursUsage (int): The 24 hours usage of the organization
            last7DaysUsage (int): The 7 days usage of the organization
            currMonthAvailPoints (int): The current month's available compute points
            currMonthUsage (int): The current month's usage compute points
            lastThrottlePopUp (str): The last time the organization was throttled
            alwaysDisplay (bool): Whether to always display the compute point toggle
            freeTierTotal (int): The initial free tier compute point allocation
            freeTierExpiresAt (str): The ISO timestamp when the free tier expires
    """

    def __init__(self, client, updatedAt=None, last24HoursUsage=None, last7DaysUsage=None, currMonthAvailPoints=None, currMonthUsage=None, lastThrottlePopUp=None, alwaysDisplay=None, freeTierTotal=None, freeTierExpiresAt=None):
        super().__init__(client, None)
        self.updated_at = updatedAt
        self.last_24_hours_usage = last24HoursUsage
        self.last_7_days_usage = last7DaysUsage
        self.curr_month_avail_points = currMonthAvailPoints
        self.curr_month_usage = currMonthUsage
        self.last_throttle_pop_up = lastThrottlePopUp
        self.always_display = alwaysDisplay
        self.free_tier_total = freeTierTotal
        self.free_tier_expires_at = freeTierExpiresAt
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'updated_at': repr(self.updated_at), f'last_24_hours_usage': repr(self.last_24_hours_usage), f'last_7_days_usage': repr(self.last_7_days_usage), f'curr_month_avail_points': repr(self.curr_month_avail_points), f'curr_month_usage': repr(
            self.curr_month_usage), f'last_throttle_pop_up': repr(self.last_throttle_pop_up), f'always_display': repr(self.always_display), f'free_tier_total': repr(self.free_tier_total), f'free_tier_expires_at': repr(self.free_tier_expires_at)}
        class_name = "ComputePointInfo"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'updated_at': self.updated_at, 'last_24_hours_usage': self.last_24_hours_usage, 'last_7_days_usage': self.last_7_days_usage, 'curr_month_avail_points': self.curr_month_avail_points,
                'curr_month_usage': self.curr_month_usage, 'last_throttle_pop_up': self.last_throttle_pop_up, 'always_display': self.always_display, 'free_tier_total': self.free_tier_total, 'free_tier_expires_at': self.free_tier_expires_at}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class PredictionFeatureGroup(AbstractApiClass):
    """
        Batch Input Feature Group

        Args:
            client (ApiClient): An authenticated API Client instance
            featureGroupId (str): The unique identifier of the feature group
            featureGroupVersion (str): The unique identifier of the feature group version used for predictions
            datasetType (str): dataset type
            default (bool): If true, this feature group is the default feature group in the model
            required (bool): If true, this feature group is required for the batch prediction
    """

    def __init__(self, client, featureGroupId=None, featureGroupVersion=None, datasetType=None, default=None, required=None):
        super().__init__(client, None)
        self.feature_group_id = featureGroupId
        self.feature_group_version = featureGroupVersion
        self.dataset_type = datasetType
        self.default = default
        self.required = required
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'feature_group_id': repr(self.feature_group_id), f'feature_group_version': repr(
            self.feature_group_version), f'dataset_type': repr(self.dataset_type), f'default': repr(self.default), f'required': repr(self.required)}
        class_name = "PredictionFeatureGroup"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'feature_group_id': self.feature_group_id, 'feature_group_version': self.feature_group_version,
                'dataset_type': self.dataset_type, 'default': self.default, 'required': self.required}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class StudioGenCosts(AbstractApiClass):
    """
        Credit cost estimation data for all Studio models.

        Args:
            client (ApiClient): An authenticated API Client instance
            videoCosts (dict): Cost lookup tables for video generation models
            imageCosts (dict): Cost lookup tables for image generation models
            lipSyncCosts (dict): Cost lookup tables for lip sync models
            overhead (int): Fixed credit overhead per generation (agentic loop cost)
    """

    def __init__(self, client, videoCosts=None, imageCosts=None, lipSyncCosts=None, overhead=None):
        super().__init__(client, None)
        self.video_costs = videoCosts
        self.image_costs = imageCosts
        self.lip_sync_costs = lipSyncCosts
        self.overhead = overhead
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'video_costs': repr(self.video_costs), f'image_costs': repr(
            self.image_costs), f'lip_sync_costs': repr(self.lip_sync_costs), f'overhead': repr(self.overhead)}
        class_name = "StudioGenCosts"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'video_costs': self.video_costs, 'image_costs': self.image_costs,
                'lip_sync_costs': self.lip_sync_costs, 'overhead': self.overhead}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class WebServiceTriggerRun(AbstractApiClass):
    """
        A web service trigger run

        Args:
            client (ApiClient): An authenticated API Client instance
            endpoint (str): The endpoint of the web service trigger run.
            createdAt (str): The creation time of the web service trigger run.
            payload (dict): The payload of the web service trigger run.
            headers (dict): The headers of the web service trigger run.
            method (str): The method of the web service trigger run.
            responseStatus (int): The HTTP response status code.
            responseBody (str): The HTTP response body.
            error (str): Error message if the request failed.
            lifecycle (str): The lifecycle status of the run.
    """

    def __init__(self, client, endpoint=None, createdAt=None, payload=None, headers=None, method=None, responseStatus=None, responseBody=None, error=None, lifecycle=None):
        super().__init__(client, None)
        self.endpoint = endpoint
        self.created_at = createdAt
        self.payload = payload
        self.headers = headers
        self.method = method
        self.response_status = responseStatus
        self.response_body = responseBody
        self.error = error
        self.lifecycle = lifecycle
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'endpoint': repr(self.endpoint), f'created_at': repr(self.created_at), f'payload': repr(self.payload), f'headers': repr(self.headers), f'method': repr(
            self.method), f'response_status': repr(self.response_status), f'response_body': repr(self.response_body), f'error': repr(self.error), f'lifecycle': repr(self.lifecycle)}
        class_name = "WebServiceTriggerRun"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'endpoint': self.endpoint, 'created_at': self.created_at, 'payload': self.payload, 'headers': self.headers, 'method': self.method,
                'response_status': self.response_status, 'response_body': self.response_body, 'error': self.error, 'lifecycle': self.lifecycle}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



import dataclasses

from .abstract import ApiClass
from .enums import DocumentType, OcrMode


@dataclasses.dataclass
class DatasetConfig(ApiClass):
    """
    An abstract class for dataset configs

    Args:
        is_documentset (bool): Whether the dataset is a document set
    """
    is_documentset: bool = dataclasses.field(default=None)


@dataclasses.dataclass
class ParsingConfig(ApiClass):
    """
    Custom config for dataset parsing.

    Args:
        escape (str): Escape character for CSV files. Defaults to '"'.
        csv_delimiter (str): Delimiter for CSV files. Defaults to None.
        file_path_with_schema (str): Path to the file with schema. Defaults to None.
    """
    escape: str = dataclasses.field(default='"')
    csv_delimiter: str = dataclasses.field(default=None)
    file_path_with_schema: str = dataclasses.field(default=None)


@dataclasses.dataclass
class DocumentProcessingConfig(ApiClass):
    """
    Document processing configuration.

    Args:
        document_type (DocumentType): Type of document. Can be one of Text, Tables and Forms, Embedded Images, etc. If not specified, type will be decided automatically.
        highlight_relevant_text (bool): Whether to extract bounding boxes and highlight relevant text in search results. Defaults to False.
        extract_bounding_boxes (bool): Whether to perform OCR and extract bounding boxes. If False, no OCR will be done but only the embedded text from digital documents will be extracted. Defaults to False.
        ocr_mode (OcrMode): OCR mode. There are different OCR modes available for different kinds of documents and use cases. This option only takes effect when extract_bounding_boxes is True.
        use_full_ocr (bool): Whether to perform full OCR. If True, OCR will be performed on the full page. If False, OCR will be performed on the non-text regions only. By default, it will be decided automatically based on the OCR mode and the document type. This option only takes effect when extract_bounding_boxes is True.
        remove_header_footer (bool): Whether to remove headers and footers. Defaults to False. This option only takes effect when extract_bounding_boxes is True.
        remove_watermarks (bool): Whether to remove watermarks. By default, it will be decided automatically based on the OCR mode and the document type. This option only takes effect when extract_bounding_boxes is True.
        convert_to_markdown (bool): Whether to convert extracted text to markdown. Defaults to False. This option only takes effect when extract_bounding_boxes is True.
        mask_pii (bool): Whether to mask personally identifiable information (PII) in the document text/tokens. Defaults to False.
        extract_images (bool): Whether to extract images from the document e.g. diagrams in a PDF page. Defaults to False.
    """
    # NOTE: The defaults should match with clouddb.document_processing_results table defaults
    document_type: DocumentType = None
    highlight_relevant_text: bool = None
    extract_bounding_boxes: bool = False
    ocr_mode: OcrMode = OcrMode.DEFAULT
    use_full_ocr: bool = None
    remove_header_footer: bool = False
    remove_watermarks: bool = True
    convert_to_markdown: bool = False
    mask_pii: bool = False
    extract_images: bool = False

    def __post_init__(self):
        self.ocr_mode = self._detect_ocr_mode()
        if self.document_type is not None:
            if DocumentType.is_ocr_forced(self.document_type):
                self.highlight_relevant_text = True
            elif self.highlight_relevant_text is None:
                self.highlight_relevant_text = False
        if self.highlight_relevant_text is not None:
            self.extract_bounding_boxes = self.highlight_relevant_text  # Highlight_relevant text acts as a wrapper over extract_bounding_boxes

    def _detect_ocr_mode(self):
        if self.document_type is not None:
            if self.document_type == DocumentType.TEXT or self.document_type == DocumentType.SIMPLE_TEXT:
                return OcrMode.DEFAULT
            elif self.document_type == DocumentType.TABLES_AND_FORMS:
                return OcrMode.LAYOUT
            elif self.document_type == DocumentType.COMPREHENSIVE_MARKDOWN:
                return OcrMode.COMPREHENSIVE_TABLE_MD
            elif self.document_type == DocumentType.EMBEDDED_IMAGES:
                return OcrMode.SCANNED
            elif self.document_type == DocumentType.SCANNED_TEXT:
                return OcrMode.SCANNED
        if self.ocr_mode is not None:
            return self.ocr_mode
        return OcrMode.AUTO

    @classmethod
    def _get_filtered_dict(cls, config: dict):
        """Filters out default values from the config"""
        from reainternal.utils import snake_case
        return {
            k: v for k, v in config.items()
            if v is not None and v != getattr(cls, snake_case(k), None)
        }


@dataclasses.dataclass
class DatasetDocumentProcessingConfig(DocumentProcessingConfig):
    """
    Document processing configuration for dataset imports.

    Args:
        extract_bounding_boxes (bool): Whether to perform OCR and extract bounding boxes. If False, no OCR will be done but only the embedded text from digital documents will be extracted. Defaults to False.
        ocr_mode (OcrMode): OCR mode. There are different OCR modes available for different kinds of documents and use cases. This option only takes effect when extract_bounding_boxes is True.
        use_full_ocr (bool): Whether to perform full OCR. If True, OCR will be performed on the full page. If False, OCR will be performed on the non-text regions only. By default, it will be decided automatically based on the OCR mode and the document type. This option only takes effect when extract_bounding_boxes is True.
        remove_header_footer (bool): Whether to remove headers and footers. Defaults to False. This option only takes effect when extract_bounding_boxes is True.
        remove_watermarks (bool): Whether to remove watermarks. By default, it will be decided automatically based on the OCR mode and the document type. This option only takes effect when extract_bounding_boxes is True.
        convert_to_markdown (bool): Whether to convert extracted text to markdown. Defaults to False. This option only takes effect when extract_bounding_boxes is True.
        page_text_column (str): Name of the output column which contains the extracted text for each page. If not provided, no column will be created.
    """
    page_text_column: str = None


@dataclasses.dataclass
class IncrementalDatabaseConnectorConfig(ApiClass):
    """
    Config information for incremental datasets from database connectors

    Args:
        timestamp_column (str): If dataset is incremental, this is the column name of the required column in the dataset. This column must contain timestamps in descending order which are used to determine the increments of the incremental dataset.
    """
    timestamp_column: str = dataclasses.field(default=None)


@dataclasses.dataclass
class AttachmentParsingConfig(ApiClass):
    """
    Config information for parsing attachments

    Args:
        feature_group_name (str): feature group name
        column_name (str): column name
        urls (str): list of urls
    """
    feature_group_name: str = dataclasses.field(default=None)
    column_name: str = dataclasses.field(default=None)
    urls: str = dataclasses.field(default=None)



from .return_class import AbstractApiClass


class NewsSearchResult(AbstractApiClass):
    """
        A single news search result.

        Args:
            client (ApiClient): An authenticated API Client instance
            title (str): The title of the news.
            url (str): The URL of the news.
            description (str): The description of the news.
            thumbnailUrl (str): The URL of the image of the news.
            thumbnailWidth (int): The width of the image of the news.
            thumbnailHeight (int): The height of the image of the news.
            faviconUrl (str): The URL of the favicon of the news.
            datePublished (str): The date the news was published.
    """

    def __init__(self, client, title=None, url=None, description=None, thumbnailUrl=None, thumbnailWidth=None, thumbnailHeight=None, faviconUrl=None, datePublished=None):
        super().__init__(client, None)
        self.title = title
        self.url = url
        self.description = description
        self.thumbnail_url = thumbnailUrl
        self.thumbnail_width = thumbnailWidth
        self.thumbnail_height = thumbnailHeight
        self.favicon_url = faviconUrl
        self.date_published = datePublished
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'title': repr(self.title), f'url': repr(self.url), f'description': repr(self.description), f'thumbnail_url': repr(self.thumbnail_url), f'thumbnail_width': repr(
            self.thumbnail_width), f'thumbnail_height': repr(self.thumbnail_height), f'favicon_url': repr(self.favicon_url), f'date_published': repr(self.date_published)}
        class_name = "NewsSearchResult"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'title': self.title, 'url': self.url, 'description': self.description, 'thumbnail_url': self.thumbnail_url, 'thumbnail_width':
                self.thumbnail_width, 'thumbnail_height': self.thumbnail_height, 'favicon_url': self.favicon_url, 'date_published': self.date_published}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class PredictionLogRecord(AbstractApiClass):
    """
        A Record for a prediction request log.

        Args:
            client (ApiClient): An authenticated API Client instance
            requestId (str): The unique identifier of the prediction request.
            query (dict): The query used to make the prediction.
            queryTimeMs (int): The time taken to make the prediction.
            timestampMs (str): The timestamp of the prediction request.
            response (dict): The prediction response.
    """

    def __init__(self, client, requestId=None, query=None, queryTimeMs=None, timestampMs=None, response=None):
        super().__init__(client, None)
        self.request_id = requestId
        self.query = query
        self.query_time_ms = queryTimeMs
        self.timestamp_ms = timestampMs
        self.response = response
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'request_id': repr(self.request_id), f'query': repr(self.query), f'query_time_ms': repr(
            self.query_time_ms), f'timestamp_ms': repr(self.timestamp_ms), f'response': repr(self.response)}
        class_name = "PredictionLogRecord"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'request_id': self.request_id, 'query': self.query, 'query_time_ms': self.query_time_ms,
                'timestamp_ms': self.timestamp_ms, 'response': self.response}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class ProjectValidation(AbstractApiClass):
    """
        A validation result for a project

        Args:
            client (ApiClient): An authenticated API Client instance
            valid (bool): `true` if the project is valid and ready to be trained, otherwise `false`.
            datasetErrors (list[dict]): A list of errors keeping the dataset from being valid
            columnHints (dict): Hints for what to set on the columns
    """

    def __init__(self, client, valid=None, datasetErrors=None, columnHints=None):
        super().__init__(client, None)
        self.valid = valid
        self.dataset_errors = datasetErrors
        self.column_hints = columnHints
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'valid': repr(self.valid), f'dataset_errors': repr(
            self.dataset_errors), f'column_hints': repr(self.column_hints)}
        class_name = "ProjectValidation"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'valid': self.valid, 'dataset_errors': self.dataset_errors,
                'column_hints': self.column_hints}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class ModelMonitorVersionMetricData(AbstractApiClass):
    """
        Data for displaying model monitor version metric data

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): The name of the metric type
            algoName (str): The name of the algo used for the prediction metric
            featureGroupVersion (str): The prediction feature group used for analysis
            modelMonitor (str): The id of the model monitor
            modelMonitorVersion (str): The id of the model monitor version
            metricInfos (dict): Name and description for metrics
            metricNames (dict): Internal name to external name mapping
            metrics (dict): Metric name to metric data
            metricCharts (list): List of different metric charts
            otherMetrics (list): List of other metrics to optionally plot
            actualValuesSupportedForDrilldown (list): List of values support for drilldown
    """

    def __init__(self, client, name=None, algoName=None, featureGroupVersion=None, modelMonitor=None, modelMonitorVersion=None, metricInfos=None, metricNames=None, metrics=None, metricCharts=None, otherMetrics=None, actualValuesSupportedForDrilldown=None):
        super().__init__(client, None)
        self.name = name
        self.algo_name = algoName
        self.feature_group_version = featureGroupVersion
        self.model_monitor = modelMonitor
        self.model_monitor_version = modelMonitorVersion
        self.metric_infos = metricInfos
        self.metric_names = metricNames
        self.metrics = metrics
        self.metric_charts = metricCharts
        self.other_metrics = otherMetrics
        self.actual_values_supported_for_drilldown = actualValuesSupportedForDrilldown
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'algo_name': repr(self.algo_name), f'feature_group_version': repr(self.feature_group_version), f'model_monitor': repr(self.model_monitor), f'model_monitor_version': repr(self.model_monitor_version), f'metric_infos': repr(
            self.metric_infos), f'metric_names': repr(self.metric_names), f'metrics': repr(self.metrics), f'metric_charts': repr(self.metric_charts), f'other_metrics': repr(self.other_metrics), f'actual_values_supported_for_drilldown': repr(self.actual_values_supported_for_drilldown)}
        class_name = "ModelMonitorVersionMetricData"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'algo_name': self.algo_name, 'feature_group_version': self.feature_group_version, 'model_monitor': self.model_monitor, 'model_monitor_version': self.model_monitor_version, 'metric_infos': self.metric_infos,
                'metric_names': self.metric_names, 'metrics': self.metrics, 'metric_charts': self.metric_charts, 'other_metrics': self.other_metrics, 'actual_values_supported_for_drilldown': self.actual_values_supported_for_drilldown}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .point_in_time_feature_info import PointInTimeFeatureInfo
from .return_class import AbstractApiClass


class NestedFeatureSchema(AbstractApiClass):
    """
        A schema description for a nested feature

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): The unique name of the column
            featureType (str): Feature Type of the Feature
            featureMapping (str): The Feature Mapping of the feature
            dataType (str): Data Type of the Feature
            detectedFeatureType (str): The detected feature type for this feature
            sourceTable (str): The source table of the column
            pointInTimeInfo (PointInTimeFeatureInfo): Point in time information for this feature
    """

    def __init__(self, client, name=None, featureType=None, featureMapping=None, dataType=None, detectedFeatureType=None, sourceTable=None, pointInTimeInfo={}):
        super().__init__(client, None)
        self.name = name
        self.feature_type = featureType
        self.feature_mapping = featureMapping
        self.data_type = dataType
        self.detected_feature_type = detectedFeatureType
        self.source_table = sourceTable
        self.point_in_time_info = client._build_class(
            PointInTimeFeatureInfo, pointInTimeInfo)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'feature_type': repr(self.feature_type), f'feature_mapping': repr(self.feature_mapping), f'data_type': repr(
            self.data_type), f'detected_feature_type': repr(self.detected_feature_type), f'source_table': repr(self.source_table), f'point_in_time_info': repr(self.point_in_time_info)}
        class_name = "NestedFeatureSchema"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'feature_type': self.feature_type, 'feature_mapping': self.feature_mapping, 'data_type': self.data_type,
                'detected_feature_type': self.detected_feature_type, 'source_table': self.source_table, 'point_in_time_info': self._get_attribute_as_dict(self.point_in_time_info)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class RangeViolation(AbstractApiClass):
    """
        Summary of important range mismatches for a numerical feature discovered by a model monitoring instance

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): Name of feature.
            trainingMin (float): Minimum value of training distribution for the specified feature.
            trainingMax (float): Maximum value of training distribution for the specified feature.
            predictionMin (float): Minimum value of prediction distribution for the specified feature.
            predictionMax (float): Maximum value of prediction distribution for the specified feature.
            freqAboveTrainingRange (float): Frequency of prediction rows below training minimum for the specified feature.
            freqBelowTrainingRange (float): Frequency of prediction rows above training maximum for the specified feature.
    """

    def __init__(self, client, name=None, trainingMin=None, trainingMax=None, predictionMin=None, predictionMax=None, freqAboveTrainingRange=None, freqBelowTrainingRange=None):
        super().__init__(client, None)
        self.name = name
        self.training_min = trainingMin
        self.training_max = trainingMax
        self.prediction_min = predictionMin
        self.prediction_max = predictionMax
        self.freq_above_training_range = freqAboveTrainingRange
        self.freq_below_training_range = freqBelowTrainingRange
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'training_min': repr(self.training_min), f'training_max': repr(self.training_max), f'prediction_min': repr(self.prediction_min), f'prediction_max': repr(
            self.prediction_max), f'freq_above_training_range': repr(self.freq_above_training_range), f'freq_below_training_range': repr(self.freq_below_training_range)}
        class_name = "RangeViolation"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'training_min': self.training_min, 'training_max': self.training_max, 'prediction_min': self.prediction_min,
                'prediction_max': self.prediction_max, 'freq_above_training_range': self.freq_above_training_range, 'freq_below_training_range': self.freq_below_training_range}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from typing import Union

from .api_class import (
    AlertActionConfig, AlertConditionConfig, BatchPredictionArgs,
    DeploymentConversationType, FeatureGroupExportConfig, PredictionArguments
)
from .feature_group_export_config import FeatureGroupExportConfig
from .refresh_schedule import RefreshSchedule
from .return_class import AbstractApiClass


class Deployment(AbstractApiClass):
    """
        A model deployment

        Args:
            client (ApiClient): An authenticated API Client instance
            deploymentId (str): A unique identifier for the deployment.
            name (str): A user-friendly name for the deployment.
            status (str): The status of the deployment.
            description (str): A description of the deployment.
            deployedAt (str): The date and time when the deployment became active, in ISO-8601 format.
            createdAt (str): The date and time when the deployment was created, in ISO-8601 format.
            projectId (str): A unique identifier for the project this deployment belongs to.
            modelId (str): The model that is currently deployed.
            modelVersion (str): The model version ID that is currently deployed.
            featureGroupId (str): The feature group that is currently deployed.
            featureGroupVersion (str): The feature group version ID that is currently deployed.
            callsPerSecond (int): The number of calls per second the deployment can handle.
            autoDeploy (bool): A flag marking the deployment as eligible for auto deployments whenever any model in the project finishes training.
            skipMetricsCheck (bool): A flag to skip metric regression with this current deployment. This field is only relevant when auto_deploy is on
            algoName (str): The name of the algorithm that is currently deployed.
            regions (list): A list of regions that the deployment has been deployed to.
            error (str): The relevant error, if the status is FAILED.
            batchStreamingUpdates (bool): A flag marking the feature group deployment as having enabled a background process which caches streamed-in rows for quicker lookup.
            algorithm (str): The algorithm that is currently deployed.
            pendingModelVersion (dict): The model that the deployment is switching to, or being stopped.
            modelDeploymentConfig (dict): The config for which model to be deployed.
            predictionOperatorId (str): The prediction operator ID that is currently deployed.
            predictionOperatorVersion (str): The prediction operator version ID that is currently deployed.
            pendingPredictionOperatorVersion (str): The prediction operator version ID that the deployment is switching to, or being stopped.
            onlineFeatureGroupId (id): The online feature group ID that the deployment is running on
            outputOnlineFeatureGroupId (id): The online feature group ID that the deployment is outputting results to
            realtimeMonitorId (id): The realtime monitor ID of the realtime-monitor that is associated with the deployment
            runtimeConfigs (dict): The runtime configurations of a deployment which is used by some of the usecases during prediction.
            isSystemCreated (bool): Whether the deployment is system created.
            refreshSchedules (RefreshSchedule): A list of refresh schedules that indicate when the deployment will be updated to the latest model version.
            featureGroupExportConfig (FeatureGroupExportConfig): The export config (file connector or database connector information) for feature group deployment exports.
            defaultPredictionArguments (PredictionArguments): The default prediction arguments for prediction APIs
    """

    def __init__(self, client, deploymentId=None, name=None, status=None, description=None, deployedAt=None, createdAt=None, projectId=None, modelId=None, modelVersion=None, featureGroupId=None, featureGroupVersion=None, callsPerSecond=None, autoDeploy=None, skipMetricsCheck=None, algoName=None, regions=None, error=None, batchStreamingUpdates=None, algorithm=None, pendingModelVersion=None, modelDeploymentConfig=None, predictionOperatorId=None, predictionOperatorVersion=None, pendingPredictionOperatorVersion=None, onlineFeatureGroupId=None, outputOnlineFeatureGroupId=None, realtimeMonitorId=None, runtimeConfigs=None, isSystemCreated=None, refreshSchedules={}, featureGroupExportConfig={}, defaultPredictionArguments={}):
        super().__init__(client, deploymentId)
        self.deployment_id = deploymentId
        self.name = name
        self.status = status
        self.description = description
        self.deployed_at = deployedAt
        self.created_at = createdAt
        self.project_id = projectId
        self.model_id = modelId
        self.model_version = modelVersion
        self.feature_group_id = featureGroupId
        self.feature_group_version = featureGroupVersion
        self.calls_per_second = callsPerSecond
        self.auto_deploy = autoDeploy
        self.skip_metrics_check = skipMetricsCheck
        self.algo_name = algoName
        self.regions = regions
        self.error = error
        self.batch_streaming_updates = batchStreamingUpdates
        self.algorithm = algorithm
        self.pending_model_version = pendingModelVersion
        self.model_deployment_config = modelDeploymentConfig
        self.prediction_operator_id = predictionOperatorId
        self.prediction_operator_version = predictionOperatorVersion
        self.pending_prediction_operator_version = pendingPredictionOperatorVersion
        self.online_feature_group_id = onlineFeatureGroupId
        self.output_online_feature_group_id = outputOnlineFeatureGroupId
        self.realtime_monitor_id = realtimeMonitorId
        self.runtime_configs = runtimeConfigs
        self.is_system_created = isSystemCreated
        self.refresh_schedules = client._build_class(
            RefreshSchedule, refreshSchedules)
        self.feature_group_export_config = client._build_class(
            FeatureGroupExportConfig, featureGroupExportConfig)
        self.default_prediction_arguments = client._build_class(
            PredictionArguments, defaultPredictionArguments)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'deployment_id': repr(self.deployment_id), f'name': repr(self.name), f'status': repr(self.status), f'description': repr(self.description), f'deployed_at': repr(self.deployed_at), f'created_at': repr(self.created_at), f'project_id': repr(self.project_id), f'model_id': repr(self.model_id), f'model_version': repr(self.model_version), f'feature_group_id': repr(self.feature_group_id), f'feature_group_version': repr(self.feature_group_version), f'calls_per_second': repr(self.calls_per_second), f'auto_deploy': repr(self.auto_deploy), f'skip_metrics_check': repr(self.skip_metrics_check), f'algo_name': repr(self.algo_name), f'regions': repr(self.regions), f'error': repr(self.error), f'batch_streaming_updates': repr(self.batch_streaming_updates), f'algorithm': repr(self.algorithm), f'pending_model_version': repr(
            self.pending_model_version), f'model_deployment_config': repr(self.model_deployment_config), f'prediction_operator_id': repr(self.prediction_operator_id), f'prediction_operator_version': repr(self.prediction_operator_version), f'pending_prediction_operator_version': repr(self.pending_prediction_operator_version), f'online_feature_group_id': repr(self.online_feature_group_id), f'output_online_feature_group_id': repr(self.output_online_feature_group_id), f'realtime_monitor_id': repr(self.realtime_monitor_id), f'runtime_configs': repr(self.runtime_configs), f'is_system_created': repr(self.is_system_created), f'refresh_schedules': repr(self.refresh_schedules), f'feature_group_export_config': repr(self.feature_group_export_config), f'default_prediction_arguments': repr(self.default_prediction_arguments)}
        class_name = "Deployment"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'deployment_id': self.deployment_id, 'name': self.name, 'status': self.status, 'description': self.description, 'deployed_at': self.deployed_at, 'created_at': self.created_at, 'project_id': self.project_id, 'model_id': self.model_id, 'model_version': self.model_version, 'feature_group_id': self.feature_group_id, 'feature_group_version': self.feature_group_version, 'calls_per_second': self.calls_per_second, 'auto_deploy': self.auto_deploy, 'skip_metrics_check': self.skip_metrics_check, 'algo_name': self.algo_name, 'regions': self.regions, 'error': self.error, 'batch_streaming_updates': self.batch_streaming_updates, 'algorithm': self.algorithm, 'pending_model_version': self.pending_model_version, 'model_deployment_config': self.model_deployment_config,
                'prediction_operator_id': self.prediction_operator_id, 'prediction_operator_version': self.prediction_operator_version, 'pending_prediction_operator_version': self.pending_prediction_operator_version, 'online_feature_group_id': self.online_feature_group_id, 'output_online_feature_group_id': self.output_online_feature_group_id, 'realtime_monitor_id': self.realtime_monitor_id, 'runtime_configs': self.runtime_configs, 'is_system_created': self.is_system_created, 'refresh_schedules': self._get_attribute_as_dict(self.refresh_schedules), 'feature_group_export_config': self._get_attribute_as_dict(self.feature_group_export_config), 'default_prediction_arguments': self._get_attribute_as_dict(self.default_prediction_arguments)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}

    def create_webhook(self, endpoint: str, webhook_event_type: str, payload_template: dict = None):
        """
        Create a webhook attached to a given deployment ID.

        Args:
            endpoint (str): URI that the webhook will send HTTP POST requests to.
            webhook_event_type (str): One of 'DEPLOYMENT_START', 'DEPLOYMENT_SUCCESS', or 'DEPLOYMENT_FAILED'.
            payload_template (dict): Template for the body of the HTTP POST requests. Defaults to {}.

        Returns:
            Webhook: The webhook attached to the deployment.
        """
        return self.client.create_deployment_webhook(self.deployment_id, endpoint, webhook_event_type, payload_template)

    def list_webhooks(self):
        """
        List all the webhooks attached to a given deployment.

        Args:
            deployment_id (str): Unique identifier of the target deployment.

        Returns:
            list[Webhook]: List of the webhooks attached to the given deployment ID.
        """
        return self.client.list_deployment_webhooks(self.deployment_id)

    def refresh(self):
        """
        Calls describe and refreshes the current object's fields

        Returns:
            Deployment: The current object
        """
        self.__dict__.update(self.describe().__dict__)
        return self

    def describe(self):
        """
        Retrieves a full description of the specified deployment.

        Args:
            deployment_id (str): Unique string identifier associated with the deployment.

        Returns:
            Deployment: Description of the deployment.
        """
        return self.client.describe_deployment(self.deployment_id)

    def update(self, description: str = None, auto_deploy: bool = None, skip_metrics_check: bool = None):
        """
        Updates a deployment's properties.

        Args:
            description (str): The new description for the deployment.
            auto_deploy (bool): Flag to enable the automatic deployment when a new Model Version finishes training.
            skip_metrics_check (bool): Flag to skip metric regression with this current deployment. This field is only relevant when auto_deploy is on
        """
        return self.client.update_deployment(self.deployment_id, description, auto_deploy, skip_metrics_check)

    def rename(self, name: str):
        """
        Updates a deployment's name

        Args:
            name (str): The new deployment name.
        """
        return self.client.rename_deployment(self.deployment_id, name)

    def set_auto(self, enable: bool = None):
        """
        Enable or disable auto deployment for the specified deployment.

        When a model is scheduled to retrain, deployments with auto deployment enabled will be marked to automatically promote the new model version. After the newly trained model completes, a check on its metrics in comparison to the currently deployed model version will be performed. If the metrics are comparable or better, the newly trained model version is automatically promoted. If not, it will be marked as a failed model version promotion with an error indicating poor metrics performance.


        Args:
            enable (bool): Enable or disable the autoDeploy property of the deployment.
        """
        return self.client.set_auto_deployment(self.deployment_id, enable)

    def set_model_version(self, model_version: str, algorithm: str = None, model_deployment_config: dict = None):
        """
        Promotes a model version and/or algorithm to be the active served deployment version

        Args:
            model_version (str): A unique identifier for the model version.
            algorithm (str): The algorithm to use for the model version. If not specified, the algorithm will be inferred from the model version.
            model_deployment_config (dict): The deployment configuration for the model to deploy.
        """
        return self.client.set_deployment_model_version(self.deployment_id, model_version, algorithm, model_deployment_config)

    def set_feature_group_version(self, feature_group_version: str):
        """
        Promotes a feature group version to be served in the deployment.

        Args:
            feature_group_version (str): Unique string identifier for the feature group version.
        """
        return self.client.set_deployment_feature_group_version(self.deployment_id, feature_group_version)

    def set_prediction_operator_version(self, prediction_operator_version: str):
        """
        Promotes a prediction operator version to be served in the deployment.

        Args:
            prediction_operator_version (str): Unique string identifier for the prediction operator version.
        """
        return self.client.set_deployment_prediction_operator_version(self.deployment_id, prediction_operator_version)

    def start(self):
        """
        Restarts the specified deployment that was previously suspended.

        Args:
            deployment_id (str): A unique string identifier associated with the deployment.
        """
        return self.client.start_deployment(self.deployment_id)

    def stop(self):
        """
        Stops the specified deployment.

        Args:
            deployment_id (str): Unique string identifier of the deployment to be stopped.
        """
        return self.client.stop_deployment(self.deployment_id)

    def delete(self):
        """
        Deletes the specified deployment. The deployment's models will not be affected. Note that the deployments are not recoverable after they are deleted.

        Args:
            deployment_id (str): Unique string identifier of the deployment to delete.
        """
        return self.client.delete_deployment(self.deployment_id)

    def set_feature_group_export_file_connector_output(self, file_format: str = None, output_location: str = None):
        """
        Sets the export output for the Feature Group Deployment to be a file connector.

        Args:
            file_format (str): The type of export output, either CSV or JSON.
            output_location (str): The file connector (cloud) location where the output should be exported.
        """
        return self.client.set_deployment_feature_group_export_file_connector_output(self.deployment_id, file_format, output_location)

    def set_feature_group_export_database_connector_output(self, database_connector_id: str, object_name: str, write_mode: str, database_feature_mapping: dict, id_column: str = None, additional_id_columns: list = None):
        """
        Sets the export output for the Feature Group Deployment to a Database connector.

        Args:
            database_connector_id (str): The unique string identifier of the database connector used.
            object_name (str): The object of the database connector to write to.
            write_mode (str): The write mode to use when writing to the database connector, either UPSERT or INSERT.
            database_feature_mapping (dict): The column/feature pairs mapping the features to the database columns.
            id_column (str): The id column to use as the upsert key.
            additional_id_columns (list): For database connectors which support it, a list of additional ID columns to use as a complex key for upserting.
        """
        return self.client.set_deployment_feature_group_export_database_connector_output(self.deployment_id, database_connector_id, object_name, write_mode, database_feature_mapping, id_column, additional_id_columns)

    def remove_feature_group_export_output(self):
        """
        Removes the export type that is set for the Feature Group Deployment

        Args:
            deployment_id (str): The ID of the deployment for which the export type is set.
        """
        return self.client.remove_deployment_feature_group_export_output(self.deployment_id)

    def set_default_prediction_arguments(self, prediction_arguments: Union[dict, PredictionArguments], set_as_override: bool = False):
        """
        Sets the deployment config.

        Args:
            prediction_arguments (PredictionArguments): The prediction arguments to set.
            set_as_override (bool): If True, use these arguments as overrides instead of defaults for predict calls

        Returns:
            Deployment: description of the updated deployment.
        """
        return self.client.set_default_prediction_arguments(self.deployment_id, prediction_arguments, set_as_override)

    def get_prediction_logs_records(self, limit: int = 10, last_log_request_id: str = '', last_log_timestamp: int = None):
        """
        Retrieves the prediction request IDs for the most recent predictions made to the deployment.

        Args:
            limit (int): The number of prediction log entries to retrieve up to the specified limit.
            last_log_request_id (str): The request ID of the last log entry to retrieve.
            last_log_timestamp (int): A Unix timestamp in milliseconds specifying the timestamp for the last log entry.

        Returns:
            list[PredictionLogRecord]: A list of prediction log records.
        """
        return self.client.get_prediction_logs_records(self.deployment_id, limit, last_log_request_id, last_log_timestamp)

    def create_alert(self, alert_name: str, condition_config: Union[dict, AlertConditionConfig], action_config: Union[dict, AlertActionConfig]):
        """
        Create a deployment alert for the given conditions.

        Only support batch prediction usage now.


        Args:
            alert_name (str): Name of the alert.
            condition_config (AlertConditionConfig): Condition to run the actions for the alert.
            action_config (AlertActionConfig): Configuration for the action of the alert.

        Returns:
            MonitorAlert: Object describing the deployment alert.
        """
        return self.client.create_deployment_alert(self.deployment_id, alert_name, condition_config, action_config)

    def list_alerts(self):
        """
        List the monitor alerts associated with the deployment id.

        Args:
            deployment_id (str): Unique string identifier for the deployment.

        Returns:
            list[MonitorAlert]: An array of deployment alerts.
        """
        return self.client.list_deployment_alerts(self.deployment_id)

    def create_realtime_monitor(self, realtime_monitor_schedule: str = None, lookback_time: int = None):
        """
        Real time monitors compute and monitor metrics of real time prediction data.

        Args:
            realtime_monitor_schedule (str): The cron expression for triggering monitor.
            lookback_time (int): Lookback time (in seconds) for each monitor trigger

        Returns:
            RealtimeMonitor: Object describing the real-time monitor.
        """
        return self.client.create_realtime_monitor(self.deployment_id, realtime_monitor_schedule, lookback_time)

    def get_conversation_response(self, message: str, deployment_token: str, deployment_conversation_id: str = None, external_session_id: str = None, llm_name: str = None, num_completion_tokens: int = None, system_message: str = None, temperature: float = 0.0, filter_key_values: dict = None, search_score_cutoff: float = None, chat_config: dict = None, doc_infos: list = None, user_info: dict = None, execute_usercode_tool: bool = False):
        """
        Return a conversation response which continues the conversation based on the input message and deployment conversation id (if exists).

        Args:
            message (str): A message from the user
            deployment_token (str): A token used to authenticate access to deployments created in this project. This token is only authorized to predict on deployments in this project, so it is safe to embed this model inside of an application or website.
            deployment_conversation_id (str): The unique identifier of a deployment conversation to continue. If not specified, a new one will be created.
            external_session_id (str): The user supplied unique identifier of a deployment conversation to continue. If specified, we will use this instead of a internal deployment conversation id.
            llm_name (str): Name of the specific LLM backend to use to power the chat experience
            num_completion_tokens (int): Default for maximum number of tokens for chat answers
            temperature (float): The generative LLM temperature
            filter_key_values (dict): A dictionary mapping column names to a list of values to restrict the retrived search results.
            search_score_cutoff (float): Cutoff for the document retriever score. Matching search results below this score will be ignored.
            chat_config (dict): A dictionary specifiying the query chat config override.
            doc_infos (list): An optional list of documents use for the conversation. A keyword 'doc_id' is expected to be present in each document for retrieving contents from docstore.
            execute_usercode_tool (bool): If True, will return the tool output in the response.
        """
        return self.client.get_conversation_response(self.deployment_id, message, deployment_token, deployment_conversation_id, external_session_id, llm_name, num_completion_tokens, system_message, temperature, filter_key_values, search_score_cutoff, chat_config, doc_infos, user_info, execute_usercode_tool)

    def get_conversation_response_with_binary_data(self, deployment_token: str, message: str, deployment_conversation_id: str = None, external_session_id: str = None, llm_name: str = None, num_completion_tokens: int = None, system_message: str = None, temperature: float = 0.0, filter_key_values: dict = None, search_score_cutoff: float = None, chat_config: dict = None, attachments: None = None):
        """
        Return a conversation response which continues the conversation based on the input message and deployment conversation id (if exists).

        Args:
            deployment_token (str): A token used to authenticate access to deployments created in this project. This token is only authorized to predict on deployments in this project, so it is safe to embed this model inside of an application or website.
            message (str): A message from the user
            deployment_conversation_id (str): The unique identifier of a deployment conversation to continue. If not specified, a new one will be created.
            external_session_id (str): The user supplied unique identifier of a deployment conversation to continue. If specified, we will use this instead of a internal deployment conversation id.
            llm_name (str): Name of the specific LLM backend to use to power the chat experience
            num_completion_tokens (int): Default for maximum number of tokens for chat answers
            temperature (float): The generative LLM temperature
            filter_key_values (dict): A dictionary mapping column names to a list of values to restrict the retrived search results.
            search_score_cutoff (float): Cutoff for the document retriever score. Matching search results below this score will be ignored.
            chat_config (dict): A dictionary specifiying the query chat config override.
            attachments (None): A dictionary of binary data to use to answer the queries.
        """
        return self.client.get_conversation_response_with_binary_data(self.deployment_id, deployment_token, message, deployment_conversation_id, external_session_id, llm_name, num_completion_tokens, system_message, temperature, filter_key_values, search_score_cutoff, chat_config, attachments)

    def create_batch_prediction(self, table_name: str = None, name: str = None, global_prediction_args: Union[dict, BatchPredictionArgs] = None, batch_prediction_args: Union[dict, BatchPredictionArgs] = None, explanations: bool = False, output_format: str = None, output_location: str = None, database_connector_id: str = None, database_output_config: dict = None, refresh_schedule: str = None, csv_input_prefix: str = None, csv_prediction_prefix: str = None, csv_explanations_prefix: str = None, output_includes_metadata: bool = None, result_input_columns: list = None, input_feature_groups: dict = None):
        """
        Creates a batch prediction job description for the given deployment.

        Args:
            table_name (str): Name of the feature group table to write the results of the batch prediction. Can only be specified if outputLocation and databaseConnectorId are not specified. If tableName is specified, the outputType will be enforced as CSV.
            name (str): Name of the batch prediction job.
            batch_prediction_args (BatchPredictionArgs): Batch Prediction args specific to problem type.
            output_format (str): Format of the batch prediction output (CSV or JSON).
            output_location (str): Location to write the prediction results. Otherwise, results will be stored in Abacus.AI.
            database_connector_id (str): Unique identifier of a Database Connection to write predictions to. Cannot be specified in conjunction with outputLocation.
            database_output_config (dict): Key-value pair of columns/values to write to the database connector. Only available if databaseConnectorId is specified.
            refresh_schedule (str): Cron-style string that describes a schedule in UTC to automatically run the batch prediction.
            csv_input_prefix (str): Prefix to prepend to the input columns, only applies when output format is CSV.
            csv_prediction_prefix (str): Prefix to prepend to the prediction columns, only applies when output format is CSV.
            csv_explanations_prefix (str): Prefix to prepend to the explanation columns, only applies when output format is CSV.
            output_includes_metadata (bool): If true, output will contain columns including prediction start time, batch prediction version, and model version.
            result_input_columns (list): If present, will limit result files or feature groups to only include columns present in this list.
            input_feature_groups (dict): A dict of {'<feature_group_type>': '<feature_group_id>'} which overrides the default input data of that type for the Batch Prediction. Default input data is the training data that was used for training the deployed model.

        Returns:
            BatchPrediction: The batch prediction description.
        """
        return self.client.create_batch_prediction(self.deployment_id, table_name, name, global_prediction_args, batch_prediction_args, explanations, output_format, output_location, database_connector_id, database_output_config, refresh_schedule, csv_input_prefix, csv_prediction_prefix, csv_explanations_prefix, output_includes_metadata, result_input_columns, input_feature_groups)

    def get_statistics_over_time(self, start_date: str, end_date: str):
        """
        Return basic access statistics for the given window

        Args:
            start_date (str): Timeline start date in ISO format.
            end_date (str): Timeline end date in ISO format. The date range must be 7 days or less.

        Returns:
            DeploymentStatistics: Object describing Time series data of the number of requests and latency over the specified time period.
        """
        return self.client.get_deployment_statistics_over_time(self.deployment_id, start_date, end_date)

    def describe_feature_group_row_process_by_key(self, primary_key_value: str):
        """
        Gets the feature group row process.

        Args:
            primary_key_value (str): The primary key value

        Returns:
            FeatureGroupRowProcess: An object representing the feature group row process
        """
        return self.client.describe_feature_group_row_process_by_key(self.deployment_id, primary_key_value)

    def list_feature_group_row_processes(self, limit: int = None, status: str = None):
        """
        Gets a list of feature group row processes.

        Args:
            limit (int): The maximum number of processes to return. Defaults to None.
            status (str): The status of the processes to return. Defaults to None.

        Returns:
            list[FeatureGroupRowProcess]: A list of object representing the feature group row process
        """
        return self.client.list_feature_group_row_processes(self.deployment_id, limit, status)

    def get_feature_group_row_process_summary(self):
        """
        Gets a summary of the statuses of the individual feature group processes.

        Args:
            deployment_id (str): The deployment id for the process

        Returns:
            FeatureGroupRowProcessSummary: An object representing the summary of the statuses of the individual feature group processes
        """
        return self.client.get_feature_group_row_process_summary(self.deployment_id)

    def reset_feature_group_row_process_by_key(self, primary_key_value: str):
        """
        Resets a feature group row process so that it can be reprocessed

        Args:
            primary_key_value (str): The primary key value

        Returns:
            FeatureGroupRowProcess: An object representing the feature group row process.
        """
        return self.client.reset_feature_group_row_process_by_key(self.deployment_id, primary_key_value)

    def get_feature_group_row_process_logs_by_key(self, primary_key_value: str):
        """
        Gets the logs for a feature group row process

        Args:
            primary_key_value (str): The primary key value

        Returns:
            FeatureGroupRowProcessLogs: An object representing the logs for the feature group row process
        """
        return self.client.get_feature_group_row_process_logs_by_key(self.deployment_id, primary_key_value)

    def create_conversation(self, name: str = None, external_application_id: str = None):
        """
        Creates a deployment conversation.

        Args:
            name (str): The name of the conversation.
            external_application_id (str): The external application id associated with the deployment conversation.

        Returns:
            DeploymentConversation: The deployment conversation.
        """
        return self.client.create_deployment_conversation(self.deployment_id, name, external_application_id)

    def list_conversations(self, external_application_id: str = None, conversation_type: Union[dict, DeploymentConversationType] = None, fetch_last_llm_info: bool = False, limit: int = None, search: str = None, include_org_level_conversations: bool = False):
        """
        Lists all conversations for the given deployment and current user.

        Args:
            external_application_id (str): The external application id associated with the deployment conversation. If specified, only conversations created on that application will be listed.
            conversation_type (DeploymentConversationType): The type of the conversation indicating its origin.
            fetch_last_llm_info (bool): If true, the LLM info for the most recent conversation will be fetched. Only applicable for system-created bots.
            limit (int): The number of conversations to return. Defaults to 600.
            search (str): The search query to filter conversations by title.
            include_org_level_conversations (bool): If true, includes org-level conversations (with no specific user) in the results.

        Returns:
            list[DeploymentConversation]: The deployment conversations.
        """
        return self.client.list_deployment_conversations(self.deployment_id, external_application_id, conversation_type, fetch_last_llm_info, limit, search, include_org_level_conversations)

    def create_external_application(self, name: str = None, description: str = None, logo: str = None, theme: dict = None):
        """
        Creates a new External Application from an existing ChatLLM Deployment.

        Args:
            name (str): The name of the External Application. If not provided, the name of the deployment will be used.
            description (str): The description of the External Application. This will be shown to users when they access the External Application. If not provided, the description of the deployment will be used.
            logo (str): The logo to be displayed.
            theme (dict): The visual theme of the External Application.

        Returns:
            ExternalApplication: The newly created External Application.
        """
        return self.client.create_external_application(self.deployment_id, name, description, logo, theme)

    def download_agent_attachment(self, attachment_id: str, deployment_conversation_id: str = None):
        """
        Return an agent attachment.

        Args:
            attachment_id (str): The attachment ID.
            deployment_conversation_id (str): The deployment conversation ID.
        """
        return self.client.download_agent_attachment(self.deployment_id, attachment_id, deployment_conversation_id)

    def wait_for_deployment(self, wait_states={'PENDING', 'DEPLOYING'}, timeout=900):
        """
        A waiting call until deployment is completed.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.
        """
        return self.client._poll(self, wait_states, timeout=timeout)

    def wait_for_pending_deployment_update(self, timeout=900):
        """
        A waiting call until deployment is in a stable state, that pending model switch is completed and previous model is stopped.

        Args:
            timeout (int): The waiting time given to the call to finish, if it doesn't finish by the allocated time, the call is said to be timed out.

        Returns:
            Deployment: the latest deployment object.
        """
        import time
        start_time = time.time()
        while True:
            if timeout and time.time() - start_time > timeout:
                raise TimeoutError(f'Maximum wait time of {timeout}s exceeded')
            self.refresh()
            if not self.pending_model_version and not self.pending_prediction_operator_version:
                break
            time.sleep(5)
        return self.refresh()

    def get_status(self):
        """
        Gets the status of the deployment.

        Returns:
            str: A string describing the status of a deploymet (pending, deploying, active, etc.).
        """
        return self.describe().status

    def create_refresh_policy(self, cron: str):
        """
        To create a refresh policy for a deployment.

        Args:
            cron (str): A cron style string to set the refresh time.

        Returns:
            RefreshPolicy: The refresh policy object.
        """
        return self.client.create_refresh_policy(self.name, cron, 'DEPLOYMENT', deployment_ids=[self.id])

    def list_refresh_policies(self):
        """
        Gets the refresh policies in a list.

        Returns:
            List[RefreshPolicy]: A list of refresh policy objects.
        """
        return self.client.list_refresh_policies(deployment_ids=[self.id])



from .return_class import AbstractApiClass


class WebAppDomain(AbstractApiClass):
    """
        Web App Domain

        Args:
            client (ApiClient): An authenticated API Client instance
            webAppDomainId (id): The ID of the web app domain
            hostname (str): The hostname of the web app domain
            domainType (str): The type of the web app domain
            lifecycle (str): The lifecycle of the web app domain
            nameservers (list): The nameservers of the web app domain
            dnsRecords (list): The DNS records of the web app domain
            metadata (dict): The metadata of the web app domain
            isRootDomain (bool): Whether the web app domain is a root domain
            isDeployed (bool): Whether the web app domain is deployed
    """

    def __init__(self, client, webAppDomainId=None, hostname=None, domainType=None, lifecycle=None, nameservers=None, dnsRecords=None, metadata=None, isRootDomain=None, isDeployed=None):
        super().__init__(client, webAppDomainId)
        self.web_app_domain_id = webAppDomainId
        self.hostname = hostname
        self.domain_type = domainType
        self.lifecycle = lifecycle
        self.nameservers = nameservers
        self.dns_records = dnsRecords
        self.metadata = metadata
        self.is_root_domain = isRootDomain
        self.is_deployed = isDeployed
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'web_app_domain_id': repr(self.web_app_domain_id), f'hostname': repr(self.hostname), f'domain_type': repr(self.domain_type), f'lifecycle': repr(self.lifecycle), f'nameservers': repr(
            self.nameservers), f'dns_records': repr(self.dns_records), f'metadata': repr(self.metadata), f'is_root_domain': repr(self.is_root_domain), f'is_deployed': repr(self.is_deployed)}
        class_name = "WebAppDomain"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'web_app_domain_id': self.web_app_domain_id, 'hostname': self.hostname, 'domain_type': self.domain_type, 'lifecycle': self.lifecycle,
                'nameservers': self.nameservers, 'dns_records': self.dns_records, 'metadata': self.metadata, 'is_root_domain': self.is_root_domain, 'is_deployed': self.is_deployed}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class ConstantsAutocompleteResponse(AbstractApiClass):
    """
        A dictionary of constants to be used in the autocomplete.

        Args:
            client (ApiClient): An authenticated API Client instance
            maxPendingRequests (int): The maximum number of pending requests.
            acceptanceDelay (int): The acceptance delay.
            debounceDelay (int): The debounce delay.
            recordUserAction (bool): Whether to record user action.
            validateSuggestion (bool): Whether to validate the suggestion.
            validationLinesThreshold (int): The number of lines to validate the suggestion.
            maxTrackedRecentChanges (int): The maximum number of recent file changes to track.
            diffThreshold (int): The diff operations threshold.
            derivativeThreshold (int): The derivative threshold for deletions
            defaultSurroundingLines (int): The default number of surrounding lines to include in the recently visited context.
            maxTrackedVisitChanges (int): The maximum number of recently visited ranges to track.
            selectionCooldownMs (int): The cooldown time in milliseconds for selection changes.
            viewingCooldownMs (int): The cooldown time in milliseconds for viewing changes.
            maxLines (int): The maximum number of lines to include in recently visited context.
            editCooldownMs (int): The cooldown time in milliseconds after last edit.
            scrollDebounceMs (int): The debounce time in milliseconds for scroll events.
            lspDeadline (int): The deadline in milliseconds for LSP context.
            diagnosticsThreshold (int): The max number of diagnostics to show.
            diagnosticEachThreshold (int): The max number of characters to show for each diagnostic type.
            numVsCodeSuggestions (int): The number of VS Code suggestions to show.
            minReindexingInterval (int): The minimum interval between reindexes in ms.
            minRefreshSummaryInterval (int): The minimum interval between refresh summary in ms.
            summaryBatchSize (int): The batch size for code summary in autocomplete.
            jobReorderInterval (int): The interval in ms to reorder jobs in the job queue for summary.
            stopRapidChanges (bool): Whether to stop rapid changes in autocomplete.
            delaySummaryBatches (int): The delay in ms between summary batches.
            delaySummaryBatchesRateLimit (int): The delay in ms in case of rate limit for delay summary batches.
            maxSymbolsFuzzyMatch (int): The max number of symbols to fuzzy match.
            fuzzySymbolMatchThreshold (int): The threshold for fuzzy symbol match.
            symbolsCacheUpdateInterval (int): The interval in ms to update the symbols cache.
            symbolsStorageUpdateInterval (int): The interval in ms to update the symbols storage.
            editPredictionSimilarityThreshold (int): The threshold for edit prediction similarity.
            minSearchWordLength (int): The minimum length of the word to be searched.
            maxOccurrencesPerWord (int): The maximum occurrences of a particular search word present in the file.
            maxWordsContentMatches (int): The maximum number of content matches from the client.
            editPredictionEnabled (bool): Whether to enable edit prediction.
            snapshotIntervalMs (int): The interval in ms to snapshot the file for recent file changes.
            linesForSnapshot (int): Limit of max number of lines to snapshot for recent file changes.
            embeddingConstants (codellmembeddingconstants): Embedding constants
    """

    def __init__(self, client, maxPendingRequests=None, acceptanceDelay=None, debounceDelay=None, recordUserAction=None, validateSuggestion=None, validationLinesThreshold=None, maxTrackedRecentChanges=None, diffThreshold=None, derivativeThreshold=None, defaultSurroundingLines=None, maxTrackedVisitChanges=None, selectionCooldownMs=None, viewingCooldownMs=None, maxLines=None, editCooldownMs=None, scrollDebounceMs=None, lspDeadline=None, diagnosticsThreshold=None, diagnosticEachThreshold=None, numVsCodeSuggestions=None, minReindexingInterval=None, minRefreshSummaryInterval=None, summaryBatchSize=None, jobReorderInterval=None, stopRapidChanges=None, delaySummaryBatches=None, delaySummaryBatchesRateLimit=None, maxSymbolsFuzzyMatch=None, fuzzySymbolMatchThreshold=None, symbolsCacheUpdateInterval=None, symbolsStorageUpdateInterval=None, editPredictionSimilarityThreshold=None, minSearchWordLength=None, maxOccurrencesPerWord=None, maxWordsContentMatches=None, editPredictionEnabled=None, snapshotIntervalMs=None, linesForSnapshot=None, embeddingConstants=None):
        super().__init__(client, None)
        self.max_pending_requests = maxPendingRequests
        self.acceptance_delay = acceptanceDelay
        self.debounce_delay = debounceDelay
        self.record_user_action = recordUserAction
        self.validate_suggestion = validateSuggestion
        self.validation_lines_threshold = validationLinesThreshold
        self.max_tracked_recent_changes = maxTrackedRecentChanges
        self.diff_threshold = diffThreshold
        self.derivative_threshold = derivativeThreshold
        self.default_surrounding_lines = defaultSurroundingLines
        self.max_tracked_visit_changes = maxTrackedVisitChanges
        self.selection_cooldown_ms = selectionCooldownMs
        self.viewing_cooldown_ms = viewingCooldownMs
        self.max_lines = maxLines
        self.edit_cooldown_ms = editCooldownMs
        self.scroll_debounce_ms = scrollDebounceMs
        self.lsp_deadline = lspDeadline
        self.diagnostics_threshold = diagnosticsThreshold
        self.diagnostic_each_threshold = diagnosticEachThreshold
        self.num_vs_code_suggestions = numVsCodeSuggestions
        self.min_reindexing_interval = minReindexingInterval
        self.min_refresh_summary_interval = minRefreshSummaryInterval
        self.summary_batch_size = summaryBatchSize
        self.job_reorder_interval = jobReorderInterval
        self.stop_rapid_changes = stopRapidChanges
        self.delay_summary_batches = delaySummaryBatches
        self.delay_summary_batches_rate_limit = delaySummaryBatchesRateLimit
        self.max_symbols_fuzzy_match = maxSymbolsFuzzyMatch
        self.fuzzy_symbol_match_threshold = fuzzySymbolMatchThreshold
        self.symbols_cache_update_interval = symbolsCacheUpdateInterval
        self.symbols_storage_update_interval = symbolsStorageUpdateInterval
        self.edit_prediction_similarity_threshold = editPredictionSimilarityThreshold
        self.min_search_word_length = minSearchWordLength
        self.max_occurrences_per_word = maxOccurrencesPerWord
        self.max_words_content_matches = maxWordsContentMatches
        self.edit_prediction_enabled = editPredictionEnabled
        self.snapshot_interval_ms = snapshotIntervalMs
        self.lines_for_snapshot = linesForSnapshot
        self.embedding_constants = embeddingConstants
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'max_pending_requests': repr(self.max_pending_requests), f'acceptance_delay': repr(self.acceptance_delay), f'debounce_delay': repr(self.debounce_delay), f'record_user_action': repr(self.record_user_action), f'validate_suggestion': repr(self.validate_suggestion), f'validation_lines_threshold': repr(self.validation_lines_threshold), f'max_tracked_recent_changes': repr(self.max_tracked_recent_changes), f'diff_threshold': repr(self.diff_threshold), f'derivative_threshold': repr(self.derivative_threshold), f'default_surrounding_lines': repr(self.default_surrounding_lines), f'max_tracked_visit_changes': repr(self.max_tracked_visit_changes), f'selection_cooldown_ms': repr(self.selection_cooldown_ms), f'viewing_cooldown_ms': repr(self.viewing_cooldown_ms), f'max_lines': repr(self.max_lines), f'edit_cooldown_ms': repr(self.edit_cooldown_ms), f'scroll_debounce_ms': repr(self.scroll_debounce_ms), f'lsp_deadline': repr(self.lsp_deadline), f'diagnostics_threshold': repr(self.diagnostics_threshold), f'diagnostic_each_threshold': repr(self.diagnostic_each_threshold), f'num_vs_code_suggestions': repr(self.num_vs_code_suggestions), f'min_reindexing_interval': repr(
            self.min_reindexing_interval), f'min_refresh_summary_interval': repr(self.min_refresh_summary_interval), f'summary_batch_size': repr(self.summary_batch_size), f'job_reorder_interval': repr(self.job_reorder_interval), f'stop_rapid_changes': repr(self.stop_rapid_changes), f'delay_summary_batches': repr(self.delay_summary_batches), f'delay_summary_batches_rate_limit': repr(self.delay_summary_batches_rate_limit), f'max_symbols_fuzzy_match': repr(self.max_symbols_fuzzy_match), f'fuzzy_symbol_match_threshold': repr(self.fuzzy_symbol_match_threshold), f'symbols_cache_update_interval': repr(self.symbols_cache_update_interval), f'symbols_storage_update_interval': repr(self.symbols_storage_update_interval), f'edit_prediction_similarity_threshold': repr(self.edit_prediction_similarity_threshold), f'min_search_word_length': repr(self.min_search_word_length), f'max_occurrences_per_word': repr(self.max_occurrences_per_word), f'max_words_content_matches': repr(self.max_words_content_matches), f'edit_prediction_enabled': repr(self.edit_prediction_enabled), f'snapshot_interval_ms': repr(self.snapshot_interval_ms), f'lines_for_snapshot': repr(self.lines_for_snapshot), f'embedding_constants': repr(self.embedding_constants)}
        class_name = "ConstantsAutocompleteResponse"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'max_pending_requests': self.max_pending_requests, 'acceptance_delay': self.acceptance_delay, 'debounce_delay': self.debounce_delay, 'record_user_action': self.record_user_action, 'validate_suggestion': self.validate_suggestion, 'validation_lines_threshold': self.validation_lines_threshold, 'max_tracked_recent_changes': self.max_tracked_recent_changes, 'diff_threshold': self.diff_threshold, 'derivative_threshold': self.derivative_threshold, 'default_surrounding_lines': self.default_surrounding_lines, 'max_tracked_visit_changes': self.max_tracked_visit_changes, 'selection_cooldown_ms': self.selection_cooldown_ms, 'viewing_cooldown_ms': self.viewing_cooldown_ms, 'max_lines': self.max_lines, 'edit_cooldown_ms': self.edit_cooldown_ms, 'scroll_debounce_ms': self.scroll_debounce_ms, 'lsp_deadline': self.lsp_deadline, 'diagnostics_threshold': self.diagnostics_threshold, 'diagnostic_each_threshold': self.diagnostic_each_threshold, 'num_vs_code_suggestions': self.num_vs_code_suggestions, 'min_reindexing_interval': self.min_reindexing_interval,
                'min_refresh_summary_interval': self.min_refresh_summary_interval, 'summary_batch_size': self.summary_batch_size, 'job_reorder_interval': self.job_reorder_interval, 'stop_rapid_changes': self.stop_rapid_changes, 'delay_summary_batches': self.delay_summary_batches, 'delay_summary_batches_rate_limit': self.delay_summary_batches_rate_limit, 'max_symbols_fuzzy_match': self.max_symbols_fuzzy_match, 'fuzzy_symbol_match_threshold': self.fuzzy_symbol_match_threshold, 'symbols_cache_update_interval': self.symbols_cache_update_interval, 'symbols_storage_update_interval': self.symbols_storage_update_interval, 'edit_prediction_similarity_threshold': self.edit_prediction_similarity_threshold, 'min_search_word_length': self.min_search_word_length, 'max_occurrences_per_word': self.max_occurrences_per_word, 'max_words_content_matches': self.max_words_content_matches, 'edit_prediction_enabled': self.edit_prediction_enabled, 'snapshot_interval_ms': self.snapshot_interval_ms, 'lines_for_snapshot': self.lines_for_snapshot, 'embedding_constants': self.embedding_constants}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class LastSelectedBot(AbstractApiClass):
    """
        The last selected bot for the user in the organization.

        Args:
            client (ApiClient): An authenticated API Client instance
            externalApplicationId (str): The ID of the last selected bot (External Application).
            mode (str): The UI mode selection. One of 'chat', 'auto', 'agent', 'custom_bot'.
            createdAt (str): Timestamp when the record was first created.
            updatedAt (str): Timestamp when the record was last modified.
    """

    def __init__(self, client, externalApplicationId=None, mode=None, createdAt=None, updatedAt=None):
        super().__init__(client, None)
        self.external_application_id = externalApplicationId
        self.mode = mode
        self.created_at = createdAt
        self.updated_at = updatedAt
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'external_application_id': repr(self.external_application_id), f'mode': repr(
            self.mode), f'created_at': repr(self.created_at), f'updated_at': repr(self.updated_at)}
        class_name = "LastSelectedBot"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'external_application_id': self.external_application_id,
                'mode': self.mode, 'created_at': self.created_at, 'updated_at': self.updated_at}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass
from .stt_gen_model import SttGenModel


class SttGenSettings(AbstractApiClass):
    """
        STT generation settings

        Args:
            client (ApiClient): An authenticated API Client instance
            settings (dict): The settings for each model.
            model (SttGenModel): Dropdown for models available for STT generation.
    """

    def __init__(self, client, settings=None, model={}):
        super().__init__(client, None)
        self.settings = settings
        self.model = client._build_class(SttGenModel, model)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'settings': repr(
            self.settings), f'model': repr(self.model)}
        class_name = "SttGenSettings"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'settings': self.settings,
                'model': self._get_attribute_as_dict(self.model)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class AnnotationConfig(AbstractApiClass):
    """
        Annotation config for a feature group

        Args:
            client (ApiClient): An authenticated API Client instance
            featureAnnotationConfigs (list): List of feature annotation configs
            labels (list): List of labels
            statusFeature (str): Name of the feature that contains the status of the annotation (Optional)
            commentsFeatures (list): Features that contain comments for the annotation (Optional)
            metadataFeature (str): Name of the feature that contains the metadata for the annotation (Optional)
    """

    def __init__(self, client, featureAnnotationConfigs=None, labels=None, statusFeature=None, commentsFeatures=None, metadataFeature=None):
        super().__init__(client, None)
        self.feature_annotation_configs = featureAnnotationConfigs
        self.labels = labels
        self.status_feature = statusFeature
        self.comments_features = commentsFeatures
        self.metadata_feature = metadataFeature
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'feature_annotation_configs': repr(self.feature_annotation_configs), f'labels': repr(self.labels), f'status_feature': repr(
            self.status_feature), f'comments_features': repr(self.comments_features), f'metadata_feature': repr(self.metadata_feature)}
        class_name = "AnnotationConfig"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'feature_annotation_configs': self.feature_annotation_configs, 'labels': self.labels,
                'status_feature': self.status_feature, 'comments_features': self.comments_features, 'metadata_feature': self.metadata_feature}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}
